/*
 * Copyright (c) 2004-2006 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-getgroup.c,v 1.8 2011/03/08 16:11:49 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/test.h"
#include "sm/io.h"
#include "sm/pwd.h"
#include "sm/grp.h"
#include "sm/sysexits.h"

static int Verbose = 0;

static void
sm_getgr(char *group, bool members)
{
	struct group *grp;

	grp = getgrnam(group);
	SM_TEST(grp != NULL);
	if (NULL == grp) {
		if (Verbose)
			sm_io_fprintf(smioerr, "errno=%d\n", errno);
		return;
	}
	if (members) {
		int i;
		char *mem;

		for (i = 0; (mem = grp->gr_mem[i]) != NULL; ++i) 
			sm_io_fprintf(smioout, "%s\n", mem);
	}
	else
		sm_io_fprintf(smioout, "%d\n", grp->gr_gid);
	return;
}

static void
sm_getpw(char *name, bool gid)
{
	struct passwd *pw;

	pw = getpwnam(name);
	SM_TEST(pw != NULL);
	if (NULL == pw) {
		if (Verbose)
			sm_io_fprintf(smioerr, "errno=%d\n", errno);
		return;
	}
	if (gid)
		sm_io_fprintf(smioout, "%d\n", pw->pw_gid);
	else
		sm_io_fprintf(smioout, "%d\n", pw->pw_uid);
	return;
}

static void
usage(const char *prg)
{
	sm_io_fprintf(smioerr, "usage: %s [options] ids\n", prg);
	sm_io_fprintf(smioerr, "-g   show gid instead of uid [for -p]\n");
	sm_io_fprintf(smioerr, "-m   show group members\n");
	sm_io_fprintf(smioerr, "-p   invoke getpwnam() instead of getgrnam()\n");
}

int
main(int argc, char *argv[])
{
	int c;
	bool pw, gid, members;
	char *name;

	pw = gid = members = false;
	while ((c = getopt(argc, argv, "gmpV")) != -1) {
		switch (c) {
		  case 'g':
			gid = true;
			break;
		  case 'm':
			members = true;
			break;
		  case 'p':
			pw = true;
			break;
		  case 'V':
			++Verbose;
			break;
		  default:
			usage(argv[0]);
			return EX_USAGE;
		}
	}

	sm_test_begin(argc, argv, "test getgrnam/getpwnam");
	argc -= optind;
	argv += optind;
	for (c = 0; c < argc; c++) {
		name = argv[c];
		if (SM_IS_EMPTY_S(name))
			continue;
		if (pw)
			sm_getpw(name, gid);
		else
			sm_getgr(name, members);
	}
	sm_io_flush(smioout);
	return sm_test_end();
}
