#!/bin/sh
# $Id: t-pm-hdrmod-4.sh,v 1.7 2011/02/11 19:29:47 ca Exp $
#
# ----------------------------------------
# test header modifications: remove, replace header (created by pmilter)
# uses smtpcnf.sh.
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

chkcto smtps MTA_USE_PMILTER '(--enable-pmilter)'

OUT="t-pm-hdrmod-4.out"
OK="t-pm-hdrmod-4.ok"
IN="t-pm-hdrmod-4.in"

PMPRG="-p ../libpmilter/t-pmilter-1"
PMOPT=""

if test X"`uname`" != "XOSF1"
then
while getopts O:p:s:V FLAG
do
  case "${FLAG}" in
    O) PMOPT="${PMOPT} ${OPTARG}";;
    p) PMPRG="-p ${OPTARG}";;
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || myexit 1
rm -f ${MTMAP}

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
smtpc {lmtp_rcpts_per_transaction_max=1;}
}
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }

smtps {
Log_Level = 19;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
policy_milter { socket {type=unix; path="pmilter.sock";} timeout=2; }
}

EOF

# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
user:	local:
rcpt1:	local:
rcpt2:	local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}

${CAT} > ${IN} <<EOF
X-Hdr1: from one
X-Date: Sun, 27 Jan 2008 01:44:29 +0200
From: Sender Name <sender@example.com>
X-Foo: bar
Subject: old

body
EOF

# ----------------------------------------
# delete, replace header
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -3 lmtpsock ${PMPRG} ${PMOPT} -P "-D 4 -e 5=Subject:_new" -Z 'smtpc2' -c "-E -D ${IN}" -E 0 -A 1 -X 0 "$@" -S"a@b.c" -R"user@local.dom"
then
  ERR=false
  if test -s ${B1}
  then
     ${EGREP} -v '^Date: ' ${B1} |\
	 ${SED} -e '/^Received:/,/^	id /d' \
		-e 's/
$//' -e 's/Mailer-Daemon@.*/Mailer-Daemon@/' > ${OUT}

     ${CAT} > ${OK} <<EOF
Return-Path: <a@b.c>
X-Hdr1: from one
X-Date: Sun, 27 Jan 2008 01:44:29 +0200
From: Sender Name <sender@example.com>
Subject: new

body

.
EOF
    if ${DIFF} -w ${OK} ${OUT} >/dev/null
    then
      :
    else
      ERR=true
    fi
  else
     ERR=true
  fi

else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi


# ----------------------------------------
# replace, delete header
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -3 lmtpsock ${PMPRG} ${PMOPT} -P "-e 5=Subject:_new -D 4" -Z 'smtpc2' -c "-E -D ${IN}" -E 0 -A 1 -X 0 "$@" -S"a@b.c" -R"user@local.dom"
then
  ERR=false
  if test -s ${B1}
  then
     ${EGREP} -v '^Date: ' ${B1} |\
	 ${SED} -e '/^Received:/,/^	id /d' \
		-e 's/
$//' -e 's/Mailer-Daemon@.*/Mailer-Daemon@/' > ${OUT}

     ${CAT} > ${OK} <<EOF
Return-Path: <a@b.c>
X-Hdr1: from one
X-Date: Sun, 27 Jan 2008 01:44:29 +0200
From: Sender Name <sender@example.com>
Subject: new

body

.
EOF
    if ${DIFF} -w ${OK} ${OUT} >/dev/null
    then
      :
    else
      ERR=true
    fi
  else
     ERR=true
  fi

else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi



# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  rm -f ${OK} ${OUT} ${IN}
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  myexit 1
fi
