#!/bin/sh
# $Id: t-rcptcnf-1.sh,v 1.5 2011/05/30 15:33:01 ca Exp $
#
# Copyright (c) 2010 Claus Assmann
#
# By using this file, you agree to the terms and conditions set
# forth in the license/LICENSE.3C file which can be found at the
# top level of this source code distribution.
#
# ----------------------------------------
# test smtps_rcpt_conf: greylisting, delay_greylisting_error_until_DATA
# uses smtpcnf.sh.
# ----------------------------------------
#

# this is a "slow" test?
if test X"${MTA_NO_SLOW_TEST}" != "X"
then
  echo "SKIPPED: $0: MTA_NO_SLOW_TEST is set"
  exit 0
fi

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

SRCPRG=${STE}/smtpc2

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || exit 1

# create empty access map
rm -f ${ACCMAP}
${MM} -t'	' -F ${ACCMAP} <<EOF
smtps_rcpt_conf:grey@local.domain	flags = greylisting;
EOF

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 14; nameserver = ${NS};
greylisting { grey_wait=10s; grey_expire=30s; white_timeout=15s; }
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 14;
flags = {access,rcpt_conf,delay_greylisting_error_until_DATA};
daemon_address = localhost:${SRVPORT};
relay_from = "no.host";
relay_to = "local.host";
wait_for_server=4;
}
EOF

# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
user:	local:
grey:	local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}

${CAT} > ${MT} <<EOF
local.host	[127.0.0.1]
y.z		[127.0.0.1]
local.domain	lmtp:
EOF

# ----------------------------------------
# try to send a message to two recipients one of which uses greylisting:
# greylisted twice, then accepted
# no mail will be sent due to delay_greylisting_error_until_DATA
# even though only one of the RCPT addresses should use greylisting.
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
rm -f grey_grey_m.db grey_grey_s.db
ERR=false
SENDER=sender${TEST}@y.z
RCPT="-Rgrey@local.domain -Ruser@local.domain"

LMTPPID=""
LMTPSOCK=lmtpsock
# start SMTP sink
${STE}/smtps2 -L unix:${LMTPSOCK} > ${LMTPL} 2>&1 &
status=$?
LMTPPID=$!
if test "x${status}" != "x0"
then
  echo "$0: FAIL: cannot start server ${status}"
  exit ${status}
fi

if ${SHELL} ${SD}/smtpcnf.sh -a '-d 6' -z -S${SENDER} ${RCPT} -E 0 -X 0 -A 0
then
  sleep 1
  if ${GREP} ERROR a0.log s0.log
  then
    ERR=true
    ${VERBOSE} && echo "$0: test ${TEST}: found ERROR in log"
  fi

  if ${GREP} "grey@local\.domain.*, stat=0$" ${SSL} > /dev/null
  then
    :
  else
    ERR=true
    ${VERBOSE} && echo "$0: test ${TEST}: 1: missing stat=0 for grey"
  fi
  if ${GREP} "user@local\.domain.*, stat=0$" ${SSL} > /dev/null
  then
    :
  else
    ERR=true
    ${VERBOSE} && echo "$0: test ${TEST}: 1: missing stat=0 for user"
  fi
  ${ERR} test X"${MTA_STOPONERROR}" != X && exit 1

  if ${SRCPRG} -r ${H}:${SRVPORT} -f"${SENDER}" ${RCPT} > ${CLTL} 2>&1
  then
    # check that no mail has been sent
    if ${GREP} "total=0 " ${CLTL} >/dev/null
    then
 
      CNT=`${GREP} -c "grey@local\.domain.*, stat=0$" ${SSL}`
      if test ${CNT} != 2
      then
        ERR=true
        ${VERBOSE} && echo "$0: test ${TEST}: 2: missing stat=0 for grey"
      fi
      CNT=`${GREP} -c "user@local\.domain.*, stat=0$" ${SSL}`
      if test ${CNT} != 2
      then
        ERR=true
        ${VERBOSE} && echo "$0: test ${TEST}: 2: missing stat=0 for user"
      fi

    else
      ${VERBOSE} && echo "$0: test ${TEST}: 2: missing total=0 in log"
      ERR=true
    fi
    if ${GREP} ERROR a0.log s0.log
    then
      ERR=true
      ${VERBOSE} && echo "$0: test ${TEST}: found ERROR in log"
    fi
  else
    :
  fi
  ${ERR} test X"${MTA_STOPONERROR}" != X && exit 1

  sleep 11
  if ${SRCPRG} -r ${H}:${SRVPORT} -f"${SENDER}" ${RCPT} > ${CLTL} 2>&1
  then
    # check that mail has been sent
    if ${GREP} "total=1 " ${CLTL} >/dev/null
    then
      :
    else
      ${VERBOSE} && echo "$0: test ${TEST}: 3: missing total=1 in log"
      ERR=true
    fi
    if ${GREP} ERROR a0.log s0.log
    then
      ERR=true
      ${VERBOSE} && echo "$0: test ${TEST}: found ERROR in log"
    fi

    if ${GREP} "grey@local\.domain.*, stat=0$" ${SSL} > /dev/null
    then
      :
    else
      ERR=true
      ${VERBOSE} && echo "$0: test ${TEST}: 3: missing stat=0 for grey"
    fi
    CNT=`${GREP} -c "user@local\.domain.*, stat=0$" ${SSL}`
    if test ${CNT} != 3
    then
      ERR=true
      ${VERBOSE} && echo "$0: test ${TEST}: 3: missing stat=0 for user"
    fi

  else
    :
  fi

else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
else
   ${VERBOSE} && echo "PASS: ${TEST}"
fi

echo ${LMTPPID} >> ${PIDS}
${SD}/stopmts.sh
fi


# ----------------------------------------
rm -f ${ACCMAP} ${ALIMAP} ${MTMAP}

# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
