#!/bin/sh
# $Id: t-mapip-1.sh,v 1.10 2013/11/30 01:56:24 ca Exp $
#
# Copyright (c) 2013 Claus Assmann
#
# By using this file, you agree to the terms and conditions set
# forth in the license/LICENSE.3C file which can be found at the
# top level of this source code distribution.
#
# ------------------------------------------------------------
# Description:
# Script to test lookup of IP addresses.
# Uses t-mapipr-0, t-mm
# ------------------------------------------------------------
#
test -s stop && exit 1
#

BDBIN="bdb1"
BDB="bdb1.db"
ERRS=0
PRG=./t-mapipr-0

if ${PRG} -C 2>&1 | grep MTA_NETINET6 >/dev/null
then
  :
else
  echo "SKIPPED: $0: ${PRG} not compiled with MTA_NETINET6"
  exit 0
fi

if [ X"${MTA_NO_IPV6_TEST}" != "X" ]
then
  echo "SKIPPED: $0: MTA_NO_IPV6_TEST set"
  exit 0
fi

cat > ${BDBIN} <<EOF
cltaddr:0:0:0:0:0:0:0:1	relay
cltaddr:1:0:0:0:0:0:0:f	ipv6-F:0:0:0:0:0:0:f
cltaddr:1:0:0:0:0:0:0:2	ipv6-1:0:0:0:0:0:0:2
cltaddr:1:0:0:0:0:0:0:	ipv6-2:0:0:0:0:0:0
cltaddr:1:0:0:0:0:0:	ipv6-3:0:0:0:0:0
cltaddr:1:0:0:0:0:	ipv6-4:0:0:0:0
cltaddr:1:0:0:0:	ipv6-5:0:0:0
cltaddr:1:0:0:	ipv6-6:0:0
cltaddr:1:0:	ipv6-7:0
cltaddr:1:	ipv6-8
cltaddr:fedc:ba98:7654:3210:fedc:ba98:7654:3210	ipv6-long
cltaddr:fedc:ba98:7654:3210:fedc:ba98:	ipv6-2
EOF

MAPTYPES="`${PRG} -l 0 2>/dev/null`"
for maptype in ${MAPTYPES}
do

# create map
rm -f ${BDB}
./t-mm -t'	' -T ${maptype} -F ${BDB} < ${BDBIN} || ERRS=`expr ${ERRS} + 1 `

for kv in \
	"cltaddr:fedc:ba98:7654:3210:fedc:ba98:7654:3210!ipv6-long" \
	"cltaddr:fedc:ba98:7654:3210:fedc:ba98:1:2!ipv6-2" \
	"cltaddr:fedc:ba98:7654:3210:fedc:ba98:7654:3!ipv6-2" \
	"cltaddr:1:0:0:0:0:0:0:f!ipv6-F:0:0:0:0:0:0:f" \
	"cltaddr:1:0:0:0:0:0:0:2!ipv6-1:0:0:0:0:0:0:2" \
	"cltaddr:1:0:0:0:0:0:0:1!ipv6-2:0:0:0:0:0:0" \
	"cltaddr:1:0:0:0:0:0:2:2!ipv6-3:0:0:0:0:0" \
	"cltaddr:1:0:0:0:0:3:3:3!ipv6-4:0:0:0:0" \
	"cltaddr:1:0:0:0:4:4:4:4!ipv6-5:0:0:0" \
	"cltaddr:1:0:0:5:5:5:5:5!ipv6-6:0:0" \
	"cltaddr:1:0:6:6:6:6:6:6!ipv6-7:0" \
	"cltaddr:1:7:7:7:7:7:7:7!ipv6-8" \
	"cltaddr:1:0:0:0:0:0:0:32!ipv6-2:0:0:0:0:0:0" \
	"cltaddr:1:0:0:0:0:0:43:0!ipv6-3:0:0:0:0:0" \
	"cltaddr:1:0:0:7A6:0:0:0:0!ipv6-6:0:0" \
	"cltaddr:0:0:0:0:0:0:0:1!relay"
do
  key=`echo $kv | cut -d'!' -f1`
  value=`echo $kv | cut -d'!' -f2`
  if ${PRG} -T ${maptype} "$@" "${key}" "${value}"
  then
    :
  else
    echo "$0: FAIL: ${maptype} $@ ${key} ${value}"
    ERRS=`expr ${ERRS} + 1 `
  fi
done

done

exit ${ERRS}
