/*
**  This file is derived from inetd.c
**	"@(#)from: inetd.c	8.4 (Berkeley) 4/13/94";
**  as available in the OpenBSD source tree.
**  It has been significantly modified to suit the needs as sendmail MCP.
*/

/*
 * Copyright (c) 1983, 1991, 1993, 1994
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef MCP_H
#define MCP_H 1

/* $Id: mcp.h,v 1.47 2014/02/19 03:32:47 ca Exp $ */

#include "sm/generic.h"
#include "sm/mcpdef.h"
#include "sm/servid.h"
#include "sm/sockcnf.h"

/*
**  Need parameter expansion, e.g., %p -> pid, to allow different names
**  for different invocations of the same program (e.g., name of socket
**  over which fd is passed).
**
**  Enforce order to start programs; currently simply done by listing
**  them in the right (reverse) order in the configuration file.
**
**  Need way to signal all children to stop, e.g., send a signal
**  to them or some data over a control socket. For now: pass TERM/INT/HUP
**  to all children.
**
**  Communication/control socket (same as socket to pass fd?)
**  Also listen on that socket for feedback.
**
**  Check names etc for "sanity", e.g., se_prg: only letter/digit/_?
**
**  Specify a "working" directory into which to chdir before running?
*/

#define MCP_FAIL_HARD	0
#define MCP_FAIL_SOFT	1
#define MCP_FAIL_N	2

typedef struct servtab_S	servtab_T, *servtab_P;

struct servtab_S
{
	char const	*se_prg;		/* name of service */
	char const	*se_kw;			/* keyword */
	char const	*se_title;		/* title */

	/* array of sockets */
	sockspec_T	 se_sockets[SM_LISTENERS_MAX];
#define se_socket	se_sockets[0]
	uint		 se_socket_n;

#define MCP_SOCKET_IS_TYPE(se, sock_type)	\
	((se)->se_socket.sckspc_type == (sock_type))
#define MCP_SOCKET_IS_UNIX(se)	MCP_SOCKET_IS_TYPE(se, SOCK_TYPE_UNIX)
#define MCP_SOCKET_IS_INET(se)	MCP_SOCKET_IS_TYPE(se, SOCK_TYPE_INET)
#define MCP_SOCKET_IS_INET6(se)	MCP_SOCKET_IS_TYPE(se, SOCK_TYPE_INET6)

#define se_port se_socket.sock_inet.inetsckspc_port
#define se_addr se_socket.sock_inet.inetsckspc_addr

	int			 se_listen_len;		/* backlog for listen(2) */

#define se_socket_name se_socket.sock_unix.unixsckspc_path
#define se_socket_umask se_socket.sock_unix.unixsckspc_umask
#define se_socket_user se_socket.sock_unix.unixsckspc_user
#define se_socket_group se_socket.sock_unix.unixsckspc_group

	int			 se_minchild;		/* min number of children */
	int			 se_maxchild;		/* max number of children */
#if MTA_USE_CPML
	int			 se_maxcpm;	/* max connects per IP per minute */
#endif
	int			 se_numchild;		/* current number of children */
	int			 se_faileda[MCP_FAIL_N];	/* failure counter array */
	time_t		 se_lastfaila[MCP_FAIL_N]; /* last time of failure array */
	int			 se_failed_max;	/* max allowed failure */
	char		 se_failed_max_isset;
	pid_t		*se_pids;		/* array of child pids */
	uint		*se_ids;		/* array of child ids */
	char		*se_pass_id;	/* pass id to program (if not NULL) */
	char		*se_exsock;		/* exchange socket */
	char		*se_exsock_abs;
	char		*se_exsock_alloc;
	char		*se_user;		/* user name to run as */
	char		*se_group;		/* group name to run as */
#ifdef LOGIN_CAP
	char		*se_class;	/* login class name to run with */
#endif

	/* data for restart dependencies */
	int			 se_nrestartdep;
	char		*se_restartdep[SM_MCP_MAXRESTARTDEP + 1];
	char		*se_restartdeps;

	char		*se_server;		/* server program */
	char		*se_server_name;	/* program without path */

	char		*se_workdir;	/* working directory (chdir(2)) */

#define MAXARGV 64
	char		*se_args;	/* program arguments (one string) */
	char		*se_argv[MAXARGV + 1];	/* program arguments */
	int			 se_fd;		/* open file descriptor */
	int			 se_count;	/* number started since se_time */
	uint		 se_flags;	/* see below */
	time_t		 se_setupfail;	/* time of last setup failure */
	bool		 se_logf_id;	/* use id in logfile */
	sm_sockaddr_T	 se_ctrladdr;	/* bound address */
	sockaddr_len_T	 se_ctrladdr_size;
	struct timeval	 se_time;	/* start of se_count */
	servtab_P	 se_next;	/* pointer to next entry */
};

/* for printf() as it does not deal with NULL; switch to sm_io_ ? */
#define SE_EXSOCK_PRT(se)	((se)->se_exsock_abs != NULL ? (se)->se_exsock_abs \
		  : ((se)->se_exsock != NULL ? (se)->se_exsock : "(null)"))

/* flags for service */
#define SE_FL_DISABLED	0x0001	/* service is disabled (permanently) */
#define SE_FL_WAIT		0x0002	/* wait for service to return */
#define SE_FL_PASS		0x0004	/* pass fd to service */
#define SE_FL_ACCEPT	0x0008	/* accept locally */

/* wait for request: start only when connection comes in */
#define SE_FL_W4REQ	0x0010

#define SE_FL_RESTART	0x0020	/* restart this service */
#define SE_FL_LEADER	0x0040	/* process that causes restart */
#define SE_FL_RESTARTING	0x0080	/* service is being restarting */

#define SE_FL_CHECKED	0x0100	/* looked at during merge */

#define SE_FL_LOGF_ID	0x0200	/* use id in logfile */

/* #define SE_FL_ESSENTIAL	0x0400	* process is "essential": if it doesn't run exit (NOT YET IMPLEMENTED) */
#define SE_FL_DELAYED	0x0800	/* setup failed, try again later */

#define SE_SET_FLAG(sep, fl)	(sep)->se_flags |= (fl)
#define SE_CLR_FLAG(sep, fl)	(sep)->se_flags &= ~(fl)
#define SE_IS_FLAG(sep, fl)	(((sep)->se_flags & (fl)) != 0)
#define SE_FAILED_PERM(sep)	(((sep)->se_flags & SE_FL_DISABLED) != 0)

/*
**  MCP global configuration context
*/

typedef struct mcp_cnf_S	mcp_cnf_T, *mcp_cnf_P;
struct mcp_cnf_S
{
	char		*mcp_cnf_cdb_base;
	char		*mcp_cnf_smtpcsock;
	char		*mcp_cnf_smtpssock;
	char		*mcp_cnf_smarsock;
	char		*mcp_cnf_hostname;
};

/*
**  MCP context
**  Note: the distinction between flags and status isn't very clean.
*/

#define MCP_IGN_MAX	64

typedef struct mcp_ctx_S	mcp_ctx_T, *mcp_ctx_P;
struct mcp_ctx_S
{
	uint		 mcp_status;
	uint		 mcp_flags;
	uint		 mcp_total_proc;	/* sum of se_maxchild */
	id_ctx_P	 mcp_id_ctx;
	mcp_cnf_T	 mcp_cnf;
	uint		 mcp_ign_n;
	char		*mcp_ign_se[MCP_IGN_MAX];
};


#define MCP_OPT_IS_SET(opt) ((opt) != NULL && *(opt) != SM_MCP_DEFAULT_CHAR)
void sm_exit(int _value);

void PRINTFLIKE(2, 3)
m_syslog(int _priority, const char *_fmt, ...);

sm_ret_T mcp_showversion(const char *_progname, const char *_cnf, uint _level);

#endif /* MCP_H */
