#!/bin/sh
# $Id: t-smar-4.sh,v 1.18 2013/11/13 17:18:23 ca Exp $
# Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ------------------------------------------------------------
# Description:
# Script to test CNAME/MX/A expansion for RFC2821 addresses.
# Uses smar, t-smar-2, startsmar.sh, DNS
# Note: this program relies on the meta1.org DNS server.
# ------------------------------------------------------------
#

if test X"${MTA_NO_DNS_TEST}" != "X"
then
  echo "SKIPPED: $0"
  exit 0
fi

TPRG=./t-smar-1
SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../smar
fi
. ${SD}/common.sh

if test X"`uname`" != "XOSF1"
then
while getopts a:s:V FLAG
do
  case "${FLAG}" in
    a) SMAROPTS="${OPTARG}";;
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

rm -f ${SOCK} ${PIDS}
# start SMAR
${SHELL} ${SD}/startsmar.sh ${SMAROPTS} || exit 1
rm -f ${OUT} ${IP} ${OK}

# mx2cname1.meta1.org.		IN	MX 10	cname1.meta1.org.
# mx2cname2cname1.meta1.org.	IN	MX 10	cname2cname1.meta1.org.
# mx2cname3cname2.meta1.org.	IN	MX 10	cname3cname2.meta1.org.
# mx2cname2itself.meta1.org.	IN	MX 10	cname2itself.meta1.org.
# mx2cnameloop1.meta1.org.	IN	MX 10	cnameloop1.meta1.org.
# mx2cnamemx.meta1.org.		IN	MX 10	cnamemx.meta1.org.
# ; cname pointing to a real name
# cname1.meta1.org.               21600   IN      CNAME   localhost.meta1.org.
# ; cname pointing to a cname
# cname2cname1.meta1.org. 21600   IN      CNAME   cname1.meta1.org.
# ; cname pointing to a cname pointing to a cname
# cname3cname2.meta1.org. 21600   IN      CNAME   cname2cname1.meta1.org.
# ; cname pointing to a cname pointing to a cname pointing to a cname
# cname4cname3.meta1.org. 21600   IN      CNAME   cname3cname2.meta1.org.
# ; cname pointing to itself
# cname2itself.meta1.org. 21600   IN      CNAME   cname2itself.meta1.org.
# ; cnames pointing to each other
# cnameloop1.meta1.org.   21600   IN      CNAME   cnameloop2.meta1.org.
# cnameloop2.meta1.org.   21600   IN      CNAME   cnameloop1.meta1.org.
# ; cname pointing to MX
# cnamemx.meta1.org.      21600   IN      CNAME   mxok.meta1.org.


TEST=1
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
# ----------------------------------------
${VALGRIND} ${TPRG} -VVVV '<a@cname1.meta1.org>' > ${OUT} 2>&1
grep '^IP' ${OUT} > ${IP}
cat > ${OK} <<EOF
IP[0]=127.0.0.1 [7f000001]
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi
fi

TEST=2
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
# ----------------------------------------
${VALGRIND} ${TPRG} -VVVV '<a@cname2cname1.meta1.org>' > ${OUT} 2>&1
grep '^IP' ${OUT} > ${IP}
cat > ${OK} <<EOF
IP[0]=127.0.0.1 [7f000001]
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi
fi

TEST=3
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
# ----------------------------------------
${VALGRIND} ${TPRG} -VVVV '<a@cnamemx.meta1.org>' > ${OUT} 2>&1
grep '^IP' ${OUT} > ${IP}
cat > ${OK} <<EOF
IP[0]=127.0.0.1 [7f000001]
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi
fi

TEST=4
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
# ----------------------------------------
${VALGRIND} ${TPRG} -VVVV '<a@mx2cname1.meta1.org>' > ${OUT} 2>&1
grep '^IP' ${OUT} > ${IP}
cat > ${OK} <<EOF
IP[0]=127.0.0.1 [7f000001]
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi
fi

TEST=5
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
# ----------------------------------------
${VALGRIND} ${TPRG} -VVVV '<a@mx2cname2cname1.meta1.org>' > ${OUT} 2>&1
grep '^IP' ${OUT} > ${IP}
cat > ${OK} <<EOF
IP[0]=127.0.0.1 [7f000001]
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi
fi

TEST=6
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
# ----------------------------------------
${VALGRIND} ${TPRG} -VVVV '<a@mx2cname3cname2.meta1.org>' > ${OUT} 2>&1
grep '^IP' ${OUT} > ${IP}
cat > ${OK} <<EOF
IP[0]=127.0.0.1 [7f000001]
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi
fi

TEST=7
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
# ----------------------------------------
# tempfail? it's a loop...
${VALGRIND} ${TPRG} -VVVV '<a@mx2cname2itself.meta1.org>' > ${OUT} 2>&1
egrep '^(IP|WARN)' ${OUT} > ${IP}
cat > ${OK} <<EOF
WARN: lookup=temp_fail
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi
fi

TEST=8
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
# ----------------------------------------
# tempfail? it's a loop...
${VALGRIND} ${TPRG} -VVVV '<a@mx2cnameloop1.meta1.org>' > ${OUT} 2>&1
egrep '^(IP|WARN)' ${OUT} > ${IP}
cat > ${OK} <<EOF
WARN: lookup=temp_fail
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi
fi

TEST=9
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
# ----------------------------------------
${VALGRIND} ${TPRG} -VVVV '<a@mx2cnamemx.meta1.org>' > ${OUT} 2>&1
egrep '^(IP|WARN)' ${OUT} > ${IP}
cat > ${OK} <<EOF
WARN: lookup=notfound
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi
fi


test ${ERRS} -eq 0 && rm -f ${OUT} ${IP} ${OK}

# stop SMAR
for i in `cat ${PIDS}`
do
  kill ${i}
done
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi

exit ${ERRS}
